==> wiki3_5_6dev2.zip: wiki3_5_6dev2/lib/Util.pm <==
################################################################################
# <p>
#   FSWikiΤǻѤ桼ƥƥؿ󶡤⥸塼Ǥ
# </p>
################################################################################
package Util;
use strict;
#===============================================================================
# <p>
#   Ϥ줿ʸURL󥳡ɤ֤ޤ
# </p>
# <pre>
# $str = Util::url_encode($str)
# </pre>
#===============================================================================
sub url_encode {
	my $retstr = shift;
	$retstr =~ s/([^ 0-9A-Za-z])/sprintf("%%%.2X", ord($1))/eg;
	$retstr =~ tr/ /+/;
	return $retstr;
}

#===============================================================================
# <p>
#   Ϥ줿ʸURLǥɤ֤ޤ
# </p>
# <pre>
# $str = Util::url_decode($str);
# </pre>
#===============================================================================
sub url_decode{
	my $retstr = shift;
	$retstr =~ tr/+/ /;
	$retstr =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
	return $retstr;
}

#===============================================================================
# <p>
#   ǥ쥯ȥꡢե̾ĥҤ礷ƥե̾ޤ
# </p>
# <pre>
# my $filename = Util::make_filename(ǥ쥯ȥ̾,ե̾,ĥ);
# </pre>
#===============================================================================
sub make_filename {
	my $dir  = shift;
	my $file = shift;
	my $ext  = shift;
	
	return $dir."/".$file.".".$ext;
}

#===============================================================================
# <p>
#   Ϥ줿ʸHTML򥨥פ֤ޤ
# </p>
# <pre>
# $str = Util::escapeHTML($str);
# </pre>
#===============================================================================
sub escapeHTML {
	my($retstr) = shift;
	my %table = (
		'&' => '&amp;',
		'"' => '&quot;',
		'<' => '&lt;',
		'>' => '&gt;',
	);
	$retstr =~ s/([&\"<>])/$table{$1}/go;
	$retstr =~ s/&amp;#([0-9]{1,5});/&#$1;/go;
	$retstr =~ s/&amp;([a-zA-Z0-9]{2,8});/&$1;/go;
	return $retstr;
}


#===============================================================================
# <p>
#   դ&quot;yyyyǯmmdd hhmiʬss&quot;˥եޥåȤޤ
# </p>
# <pre>
# my $date = Util::format_date(time());
# </pre>
#===============================================================================
sub format_date {
	my $t = shift;
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($t);
	return sprintf("%04dǯ%02d%02d %02d%02dʬ%02d",
	               $year+1900,$mon+1,$mday,$hour,$min,$sec);
}

#===============================================================================
# <p>
#   ʸξüζڤȤޤ
# </p>
# <pre>
# $text = Util::trim($text);
# </pre>
#===============================================================================
sub trim {
	my $text = shift;
	if(!defined($text)){
		return "";
	}
	$text =~ s/^(?:\s)+//o;
	$text =~ s/(?:\s)+$//o;
	return $text;
}


#===============================================================================
# <p>
#   ʸΤߤޤ
# <p>
# <pre>
# my $html = "<B>ʸ</B>";
# # &lt;B&gt;&lt;/B&gt;&quot;ʸ&quot;Τ߼
# my $text = Util::delete_tag($html);
# </pre>
#===============================================================================
sub delete_tag {
	my $text = shift;
	$text =~ s/<(.|\s)+?>//g;
	return $text;
}

#===============================================================================
# <p>
#   ͤɤåޤͤξϿǤʤϵ֤ޤ
# </p>
# <pre>
# if(Util::check_numeric($param)){
#   # ξν
# } else {
#   # Ǥʤν
# }
# </pre>
#===============================================================================
sub check_numeric {
	my $text = shift;
	if($text =~ /^[0-9]+$/){
		return 1;
	} else {
		return 0;
	}
}


#===============================================================================
# <p>
#   Ԥ˥᡼ޤ
#   setup.datƤ˱sendmailޥɤ⤷SMTP̿ˤäƥ᡼뤬ޤ
#   ɤꤵƤʤԤ鷺顼ˤʤޤ
#   SMTP硢Υ᥽åɤƤӽФNet::SMTPuseޤ
# </p>
# <pre>
# Util::send_mail($wiki,̾,ʸ);
# </pre>
#===============================================================================
sub send_mail {
	my $wiki    = shift;
	my $subject = Jcode->new(shift)->mime_encode();
	my $content = &Jcode::convert(shift,'jis');
	
	if(($wiki->config('send_mail') eq "" && $wiki->config('smtp_server') eq "") ||
	   $wiki->config('admin_mail') eq ""){
		return;
	}

	my ($sec, $min, $hour, $day, $mon, $year, $wday) = localtime(time);
	my $wday_str  = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat')[$wday];
	my $mon_str   = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')[$mon];
	my $date = sprintf("%s, %02d %s %4d %02d:%02d:%02d +0900", $wday_str, $day, $mon_str, $year+1900, $hour, $min, $sec);

	my $mail = "Subject: $subject\n".
	           "From: ".$wiki->config('admin_mail')."\n".
	           "To: ".$wiki->config('admin_mail')."\n".
	           "Date: $date\n".
	           "Content-Transfer-Encoding: 7bit\n".
	           "Content-Type: text/plain; charset=\"ISO-2022-JP\"\n".
	           "Reply-To: ".$wiki->config('admin_mail')."\n".
	           "\n".
	           $content;
	
	# sendmailޥɤ
	if($wiki->config('send_mail') ne ""){
		open(MAIL,"| ".$wiki->config('send_mail')." ".$wiki->config('admin_mail'));
		print MAIL $mail;
		close(MAIL);
		
	# Net::SMTP
	} else {
		eval("use Net::SMTP;");
		my $smtp = Net::SMTP->new($wiki->config('smtp_server'));
		$smtp->mail($wiki->config('admin_mail'));
		$smtp->to($wiki->config('admin_mail'));
		$smtp->data();
		$smtp->datasend($mail);
		$smtp->quit();
	}
}

#===============================================================================
# <p>
#   饤Ȥäɤåޤ
#   äξϿǤʤϵ֤ޤ
# </p>
# <pre>
# if(Util::handyphone()){
#   # äξν
# } else {
#   # äǤʤν
# }
# </pre>
#===============================================================================
sub handyphone {
	my $ua = $ENV{'HTTP_USER_AGENT'};
	if(!defined($ua)){
		return 0;
	}
	if($ua=~/^DoCoMo\// || $ua=~ /^J-PHONE\// || $ua=~ /UP\.Browser/ || $ua=~ /\(DDIPOCKET\;/){
		return 1;
	} else {
		return 0;
	}
}

#===============================================================================
# <p>
#   եǼǥ쥯ȥʥǥեȤǤ./configˤꤷեɤ߹ߡ
#   ϥåե󥹤ȤƼޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# my $hashref = Util::load_config_hash($wiki,&quot;hoge.dat&quot;);
# </pre>
#===============================================================================
sub load_config_hash {
	my $wiki     = shift;
	my $filename = shift;
	my $text = &load_config_text($wiki,$filename);
	my @lines = split(/\n/,$text);
	my $hash = {};
	foreach my $line (@lines){
		$line = &trim($line);
		if(index($line,"#")==0 || $line eq "\n" || $line eq "\r" || $line eq "\r\n"){
			next;
		}
		my ($name,@values) = split(/=/,$line);
		$name = &trim($name);
		my $value = &trim(join("=",@values));
		$hash->{$name} = $value;
	}
	return $hash;
}

#===============================================================================
# <p>
#   եǼǥ쥯ȥʥǥեȤǤ./configˤꤷեɤ߹ߡ
#   եƤʸȤƼޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# my $content = Util::load_config_text($wiki,&quot;hoge.dat&quot;);
# </pre>
#===============================================================================
sub load_config_text {
	my $wiki     = shift;
	my $filename = shift;
	my $fullpath = $filename;
	if(defined($wiki)){
		$fullpath = $wiki->config('config_dir')."/$filename";
	}
	
	if(defined($wiki->{config_cache}->{$fullpath})){
		return $wiki->{config_cache}->{$fullpath};
	}
	
	open(CONFIG,$fullpath) or return "";
	binmode(CONFIG);
	my $buf = "";
	while(my $line = <CONFIG>){
		$buf .= $line;
	}
	close(CONFIG);
	
	$buf =~ s/\r\n/\n/g;
	$buf =~ s/\r/\n/g;
	
	$wiki->{config_cache}->{$fullpath} = $buf;
	
	return $buf;
}

#===============================================================================
# <p>
#   Ϥϥåե󥹤եǼǥ쥯ȥʥǥեȤǤ./configˤ
#  ꤷե̾¸ޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# Util::save_config_hash(ե̾,ϥåե);
# </pre>
#===============================================================================
sub save_config_hash {
	my $wiki     = shift;
	my $filename = shift;
	my $hash     = shift;
	my $text = "";
	foreach my $key (sort(keys(%$hash))){
		$text .= $key."=".$hash->{$key}."\n";
	}
	&save_config_text($wiki,$filename,$text);
}

#===============================================================================
# <p>
#   ϤƥȤեǼǥ쥯ȥʥǥեȤǤ./configˤ
#  ꤷե̾¸ޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# Util::save_config_hash(ե̾,ƥ);
# </pre>
#===============================================================================
sub save_config_text {
	my $wiki     = shift;
	my $filename = shift;
	my $text     = shift;
	
	$text =~ s/\r\n/\n/g;
	$text =~ s/\r/\n/g;
	
	my $fullpath = $filename;
	if(defined($wiki)){
		$fullpath = $wiki->config('config_dir')."/$filename";
	}
	open(CONFIG,">$fullpath") or die $!;
	binmode(CONFIG);
	print CONFIG $text;
	close(CONFIG);
	
	$wiki->{config_cache}->{$fullpath} = $text;
}

#===============================================================================
# <p>
#   饤ץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <p>
#   Υ᥽åɤ3.4ϤȤθߴݻ뤿˻Ĥޤ
#   3.6ѻߤΤȤޤ
# </p>
#===============================================================================
sub error {
	my $message = shift;
	return "<span class=\"error\">".&Util::escapeHTML($message)."</span>";
}

#===============================================================================
# <p>
#   饤ץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <pre>
#  return Util::inline_error('ץ̾ꤵƤޤ');
# </pre>
#===============================================================================
sub inline_error {
	my $message = shift;
	my $type    = shift;
	
	if(uc($type) eq "WIKI"){
		return "<<$message>>";
	} else {
		return "<span class=\"error\">".&Util::escapeHTML($message)."</span>";
	}
}

#===============================================================================
# <p>
#   ѥ饰եץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <pre>
# return Util::paragraph_error('ץ̾ꤵƤޤ');
# </pre>
#===============================================================================
sub paragraph_error {
	my $message = shift;
	my $type    = shift;
	
	if(uc($type) eq "WIKI"){
		return "<<$message>>";
	} else {
		return "<p><span class=\"error\">".&Util::escapeHTML($message)."</span></p>";
	}
}


#===============================================================================
# <p>
#   URLGETꥯȤȯԤ쥹ݥ󥹤ΥܥǥֵѤޤ
#   δؿƤӽФLWP::UserAgentuseޤ
# </p>
# <pre>
# my $response = Util::get_response($wiki,URL);
# </pre>
#===============================================================================
sub get_response {
	my $wiki = shift;
	my $url  = shift;

	eval("use LWP::UserAgent;");
	eval("use MIME::Base64;");

	my $ua  = LWP::UserAgent->new();
	my $req = HTTP::Request->new('GET',$url);
	
	# ץ
	my $proxy_host = $wiki->config('proxy_host');
	my $proxy_port = $wiki->config('proxy_port');
	my $proxy_user = $wiki->config('proxy_user');
	my $proxy_pass = $wiki->config('proxy_pass');
	
	if($proxy_host ne "" && $proxy_port ne ""){
		$ua->proxy("http","http://$proxy_host:$proxy_port");
		if($proxy_user ne "" && $proxy_pass ne ""){
			$req->header('Proxy-Authorization'=>"Basic ".&MIME::Base64::encode("$proxy_user:$proxy_pass"));
		}
	}
	
	# ꥯȤȯ
	my $res = $ua->request($req);
	return $res->content();
}

#===============================================================================
# <p>
#   ⥸塼̾ե̾ޤ
#   㤨plugin::core::InstallϤplugin/core/Install.pmֵѤޤ
# </p>
# <pre>
# $file = Util::get_module_file(⥸塼̾);
# </pre>
#===============================================================================
sub get_module_file {
	return join('/',split(/::/,shift)).'.pm';
}

#===============================================================================
# <p>
#   ǥХådebug.logˤ򥫥ȥǥ쥯ȥ˽Ϥޤ
#   Wiki::DEBUG=1ξΤ߽ϤԤޤ
# </p>
#===============================================================================
sub debug {
	my $message = shift;
	if($Wiki::DEBUG==1){
		my $date = &Util::format_date(time());
		open(LOG,">>debug.log");
		print LOG "$date $message\n";
		close(LOG);
	}
}

#===============================================================================
# <p>
#   Digest::Perl::MD5ѤѥɤΰŹ沽Ԥޤ
#   ˥ѥɡ˥ȤϤޤ
#   Υ᥽åɤƤӽФDigest::Perl::MD5useޤ
# </p>
# <pre>
# my $md5pass = Util::md5($pass,$account);
# </pre>
#===============================================================================
sub md5 {
	my $pass = shift;
	my $salt = shift;
	
	eval("use Digest::Perl::MD5;");
	
	my $md5 = Digest::Perl::MD5->new();
	$md5->add($pass);
	$md5->add($salt);
	
	return $md5->hexdigest;
}

#===============================================================================
# <p>
#   tarեޤ
# <p>
# <pre>
# Util::compress($filename,@filelist);
# </pre>
# <p>
# @filelistˤϥ֤˴ޤե롢ޤϥǥ쥯ȥΥꥹȤꤷޤ
# ǥ쥯ȥꤷ硢۲Υե롦ǥ쥯ȥ귲ϺƵŪ˥֤
# ɲäޤδؿƤӽФArcive::Taruseޤ
# </p>
# <p>
# ޤδؿǤ¾ϹԤޤ
# Ʊ񤭹ߤ򤹤뤿ˤϳǲ餫¾,⤷ϰդʥե̾
# ֤ʤɤνԤɬפޤ
# </p>
#===============================================================================
sub create_archive {
	my $file = shift;
	my @dirs = @_;
	my @files;
	
	foreach my $dir (@dirs){
		_push_files(\@files,$dir);
	}
	
	eval("use Archive::Tar;");
	Archive::Tar->create_archive($file,0,@files);
}

#===============================================================================
# compressؿ椫ƵŪ˸ƤФprivateʴؿ
#===============================================================================
sub _push_files {
	my $list = shift;
	my $dir  = shift;
	push(@$list,$dir);
	
	if(-d $dir){
		foreach my $entry (glob("$dir/*")){
			_push_files($list,$entry);
		}
	}
}

1;

==> wiki3_5_7dev3.zip: wiki3_5_7dev3/lib/Util.pm <==
################################################################################
# <p>
#   FSWikiΤǻѤ桼ƥƥؿ󶡤⥸塼Ǥ
# </p>
################################################################################
package Util;
use strict;
#===============================================================================
# <p>
#   Ϥ줿ʸURL󥳡ɤ֤ޤ
# </p>
# <pre>
# $str = Util::url_encode($str)
# </pre>
#===============================================================================
sub url_encode {
	my $retstr = shift;
	$retstr =~ s/([^ 0-9A-Za-z])/sprintf("%%%.2X", ord($1))/eg;
	$retstr =~ tr/ /+/;
	return $retstr;
}

#===============================================================================
# <p>
#   Ϥ줿ʸURLǥɤ֤ޤ
# </p>
# <pre>
# $str = Util::url_decode($str);
# </pre>
#===============================================================================
sub url_decode{
	my $retstr = shift;
	$retstr =~ tr/+/ /;
	$retstr =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
	return $retstr;
}

#===============================================================================
# <p>
#   Cookiepath˻ꤹʸޤ
# </p>
# <pre>
# $path = Util::cookie_path($wiki);
# </pre>
#===============================================================================
sub cookie_path {
	my $wiki = shift;
	my $script_name = quotemeta($wiki->config('script_name'));
	my $path = $ENV{'REQUEST_URI'};
	$path =~ s/\?.*//;
	$path =~ s/$script_name$//;
	return $path;
}

#===============================================================================
# <p>
#   ǥ쥯ȥꡢե̾ĥҤ礷ƥե̾ޤ
# </p>
# <pre>
# my $filename = Util::make_filename(ǥ쥯ȥ̾,ե̾,ĥ);
# </pre>
#===============================================================================
sub make_filename {
	my $dir  = shift;
	my $file = shift;
	my $ext  = shift;
	
	return $dir."/".$file.".".$ext;
}

#===============================================================================
# <p>
#   Ϥ줿ʸHTML򥨥פ֤ޤ
# </p>
# <pre>
# $str = Util::escapeHTML($str);
# </pre>
#===============================================================================
sub escapeHTML {
	my($retstr) = shift;
	my %table = (
		'&' => '&amp;',
		'"' => '&quot;',
		'<' => '&lt;',
		'>' => '&gt;',
	);
	$retstr =~ s/([&\"<>])/$table{$1}/go;
	#$retstr =~ s/&amp;#([0-9]{1,5});/&#$1;/go;
	#$retstr =~ s/&amp;([a-zA-Z0-9]{2,8});/&$1;/go;
	return $retstr;
}


#===============================================================================
# <p>
#   դ&quot;yyyyǯmmdd hhmiʬss&quot;˥եޥåȤޤ
# </p>
# <pre>
# my $date = Util::format_date(time());
# </pre>
#===============================================================================
sub format_date {
	my $t = shift;
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($t);
	return sprintf("%04dǯ%02d%02d %02d%02dʬ%02d",
	               $year+1900,$mon+1,$mday,$hour,$min,$sec);
}

#===============================================================================
# <p>
#   ʸξüζڤȤޤ
# </p>
# <pre>
# $text = Util::trim($text);
# </pre>
#===============================================================================
sub trim {
	my $text = shift;
	if(!defined($text)){
		return "";
	}
	$text =~ s/^(?:\s)+//o;
	$text =~ s/(?:\s)+$//o;
	return $text;
}


#===============================================================================
# <p>
#   ʸΤߤޤ
# <p>
# <pre>
# my $html = "<B>ʸ</B>";
# # &lt;B&gt;&lt;/B&gt;&quot;ʸ&quot;Τ߼
# my $text = Util::delete_tag($html);
# </pre>
#===============================================================================
sub delete_tag {
	my $text = shift;
	$text =~ s/<(.|\s)+?>//g;
	return $text;
}

#===============================================================================
# <p>
#   ͤɤåޤͤξϿǤʤϵ֤ޤ
# </p>
# <pre>
# if(Util::check_numeric($param)){
#   # ξν
# } else {
#   # Ǥʤν
# }
# </pre>
#===============================================================================
sub check_numeric {
	my $text = shift;
	if($text =~ /^[0-9]+$/){
		return 1;
	} else {
		return 0;
	}
}


#===============================================================================
# <p>
#   Ԥ˥᡼ޤ
#   setup.datƤ˱sendmailޥɤ⤷SMTP̿ˤäƥ᡼뤬ޤ
#   ɤꤵƤʤԤ鷺顼ˤʤޤ
#   SMTP硢Υ᥽åɤƤӽФNet::SMTPuseޤ
# </p>
# <pre>
# Util::send_mail($wiki,̾,ʸ);
# </pre>
#===============================================================================
sub send_mail {
	my $wiki    = shift;
	my $subject = Jcode->new(shift)->mime_encode();
	my $content = &Jcode::convert(shift,'jis');
	
	if(($wiki->config('send_mail') eq "" && $wiki->config('smtp_server') eq "") ||
	   $wiki->config('admin_mail') eq ""){
		return;
	}

	my ($sec, $min, $hour, $day, $mon, $year, $wday) = localtime(time);
	my $wday_str  = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat')[$wday];
	my $mon_str   = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')[$mon];
	my $date = sprintf("%s, %02d %s %4d %02d:%02d:%02d +0900", $wday_str, $day, $mon_str, $year+1900, $hour, $min, $sec);

	my $mail = "Subject: $subject\n".
	           "From: ".$wiki->config('admin_mail')."\n".
	           "To: ".$wiki->config('admin_mail')."\n".
	           "Date: $date\n".
	           "Content-Transfer-Encoding: 7bit\n".
	           "Content-Type: text/plain; charset=\"ISO-2022-JP\"\n".
	           "Reply-To: ".$wiki->config('admin_mail')."\n".
	           "\n".
	           $content;
	
	# sendmailޥɤ
	if($wiki->config('send_mail') ne ""){
		open(MAIL,"| ".$wiki->config('send_mail')." ".$wiki->config('admin_mail'));
		print MAIL $mail;
		close(MAIL);
		
	# Net::SMTP
	} else {
		eval("use Net::SMTP;");
		my $smtp = Net::SMTP->new($wiki->config('smtp_server'));
		$smtp->mail($wiki->config('admin_mail'));
		$smtp->to($wiki->config('admin_mail'));
		$smtp->data();
		$smtp->datasend($mail);
		$smtp->quit();
	}
}

#===============================================================================
# <p>
#   饤Ȥäɤåޤ
#   äξϿǤʤϵ֤ޤ
# </p>
# <pre>
# if(Util::handyphone()){
#   # äξν
# } else {
#   # äǤʤν
# }
# </pre>
#===============================================================================
sub handyphone {
	my $ua = $ENV{'HTTP_USER_AGENT'};
	if(!defined($ua)){
		return 0;
	}
	if($ua=~/^DoCoMo\// || $ua=~ /^J-PHONE\// || $ua=~ /UP\.Browser/ || $ua=~ /\(DDIPOCKET\;/){
		return 1;
	} else {
		return 0;
	}
}

#===============================================================================
# <p>
#   եǼǥ쥯ȥʥǥեȤǤ./configˤꤷեɤ߹ߡ
#   ϥåե󥹤ȤƼޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# my $hashref = Util::load_config_hash($wiki,&quot;hoge.dat&quot;);
# </pre>
#===============================================================================
sub load_config_hash {
	my $wiki     = shift;
	my $filename = shift;
	my $text = &load_config_text($wiki,$filename);
	my @lines = split(/\n/,$text);
	my $hash = {};
	foreach my $line (@lines){
		$line = &trim($line);
		if(index($line,"#")==0 || $line eq "\n" || $line eq "\r" || $line eq "\r\n"){
			next;
		}
		my $name  = substr($line,0,index($line,'='));
		my $value = substr($line,index($line,'=')+1,length($line));
		$name  = &trim($name);
		$value = &trim($value);
		$hash->{$name} = $value;
	}
	return $hash;
}

#===============================================================================
# <p>
#   եǼǥ쥯ȥʥǥեȤǤ./configˤꤷեɤ߹ߡ
#   եƤʸȤƼޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# my $content = Util::load_config_text($wiki,&quot;hoge.dat&quot;);
# </pre>
#===============================================================================
sub load_config_text {
	my $wiki     = shift;
	my $filename = shift;
	my $fullpath = $filename;
	if(defined($wiki)){
		$fullpath = $wiki->config('config_dir')."/$filename";
	}
	
	if(defined($wiki->{config_cache}->{$fullpath})){
		return $wiki->{config_cache}->{$fullpath};
	}
	
	open(CONFIG,$fullpath) or return "";
	binmode(CONFIG);
	my $buf = "";
	while(my $line = <CONFIG>){
		$buf .= $line;
	}
	close(CONFIG);
	
	$buf =~ s/\r\n/\n/g;
	$buf =~ s/\r/\n/g;
	
	$wiki->{config_cache}->{$fullpath} = $buf;
	
	return $buf;
}

#===============================================================================
# <p>
#   Ϥϥåե󥹤եǼǥ쥯ȥʥǥեȤǤ./configˤ
#  ꤷե̾¸ޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# Util::save_config_hash(ե̾,ϥåե);
# </pre>
#===============================================================================
sub save_config_hash {
	my $wiki     = shift;
	my $filename = shift;
	my $hash     = shift;
	my $text = "";
	foreach my $key (sort(keys(%$hash))){
		$text .= $key."=".$hash->{$key}."\n";
	}
	&save_config_text($wiki,$filename,$text);
}

#===============================================================================
# <p>
#   ϤƥȤեǼǥ쥯ȥʥǥեȤǤ./configˤ
#  ꤷե̾¸ޤˤ$wikiϤǥե̾ꤷޤ
# </p>
# <pre>
# Util::save_config_hash(ե̾,ƥ);
# </pre>
#===============================================================================
sub save_config_text {
	my $wiki     = shift;
	my $filename = shift;
	my $text     = shift;
	
	$text =~ s/\r\n/\n/g;
	$text =~ s/\r/\n/g;
	
	my $fullpath = $filename;
	if(defined($wiki)){
		$fullpath = $wiki->config('config_dir')."/$filename";
	}
	open(CONFIG,">$fullpath") or die $!;
	binmode(CONFIG);
	print CONFIG $text;
	close(CONFIG);
	
	$wiki->{config_cache}->{$fullpath} = $text;
}

#===============================================================================
# <p>
#   饤ץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <p>
#   Υ᥽åɤ3.4ϤȤθߴݻ뤿˻Ĥޤ
#   3.6ѻߤΤȤޤ
# </p>
#===============================================================================
sub error {
	my $message = shift;
	return "<span class=\"error\">".&Util::escapeHTML($message)."</span>";
}

#===============================================================================
# <p>
#   饤ץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <pre>
#  return Util::inline_error('ץ̾ꤵƤޤ');
# </pre>
#===============================================================================
sub inline_error {
	my $message = shift;
	my $type    = shift;
	
	if(uc($type) eq "WIKI"){
		return "<<$message>>";
	} else {
		return "<span class=\"error\">".&Util::escapeHTML($message)."</span>";
	}
}

#===============================================================================
# <p>
#   ѥ饰եץ饰󤫤饨顼å֤˻ѤƤ
# </p>
# <pre>
# return Util::paragraph_error('ץ̾ꤵƤޤ');
# </pre>
#===============================================================================
sub paragraph_error {
	my $message = shift;
	my $type    = shift;
	
	if(uc($type) eq "WIKI"){
		return "<<$message>>";
	} else {
		return "<p><span class=\"error\">".&Util::escapeHTML($message)."</span></p>";
	}
}


#===============================================================================
# <p>
#   URLGETꥯȤȯԤ쥹ݥ󥹤ΥܥǥֵѤޤ
#   δؿƤӽФLWP::UserAgentuseޤ
# </p>
# <pre>
# my $response = Util::get_response($wiki,URL);
# </pre>
#===============================================================================
sub get_response {
	my $wiki = shift;
	my $url  = shift;

	eval("use LWP::UserAgent;");
	eval("use MIME::Base64;");

	my $ua  = LWP::UserAgent->new();
	my $req = HTTP::Request->new('GET',$url);
	
	# ץ
	my $proxy_host = $wiki->config('proxy_host');
	my $proxy_port = $wiki->config('proxy_port');
	my $proxy_user = $wiki->config('proxy_user');
	my $proxy_pass = $wiki->config('proxy_pass');
	
	if($proxy_host ne "" && $proxy_port ne ""){
		$ua->proxy("http","http://$proxy_host:$proxy_port");
		if($proxy_user ne "" && $proxy_pass ne ""){
			$req->header('Proxy-Authorization'=>"Basic ".&MIME::Base64::encode("$proxy_user:$proxy_pass"));
		}
	}
	
	# ꥯȤȯ
	my $res = $ua->request($req);
	return $res->content();
}

#===============================================================================
# <p>
#   ⥸塼̾ե̾ޤ
#   㤨plugin::core::InstallϤplugin/core/Install.pmֵѤޤ
# </p>
# <pre>
# $file = Util::get_module_file(⥸塼̾);
# </pre>
#===============================================================================
sub get_module_file {
	return join('/',split(/::/,shift)).'.pm';
}

#===============================================================================
# <p>
#   ǥХådebug.logˤ򥫥ȥǥ쥯ȥ˽Ϥޤ
#   Wiki::DEBUG=1ξΤ߽ϤԤޤ
# </p>
#===============================================================================
sub debug {
	my $message = shift;
	if($Wiki::DEBUG==1){
		my $date = &Util::format_date(time());
		open(LOG,">>debug.log");
		print LOG "$date $message\n";
		close(LOG);
	}
}

#===============================================================================
# <p>
#   Digest::Perl::MD5ѤѥɤΰŹ沽Ԥޤ
#   ˥ѥɡ˥ȤϤޤ
#   Υ᥽åɤƤӽФDigest::Perl::MD5useޤ
# </p>
# <pre>
# my $md5pass = Util::md5($pass,$account);
# </pre>
#===============================================================================
sub md5 {
	my $pass = shift;
	my $salt = shift;
	
	eval("use Digest::Perl::MD5;");
	
	my $md5 = Digest::Perl::MD5->new();
	$md5->add($pass);
	$md5->add($salt);
	
	return $md5->hexdigest;
}

#===============================================================================
# <p>
#   tarեޤ
# <p>
# <pre>
# Util::compress($filename,@filelist);
# </pre>
# <p>
# @filelistˤϥ֤˴ޤե롢ޤϥǥ쥯ȥΥꥹȤꤷޤ
# ǥ쥯ȥꤷ硢۲Υե롦ǥ쥯ȥ귲ϺƵŪ˥֤
# ɲäޤδؿƤӽФArcive::Taruseޤ
# </p>
# <p>
# ޤδؿǤ¾ϹԤޤ
# Ʊ񤭹ߤ򤹤뤿ˤϳǲ餫¾,⤷ϰդʥե̾
# ֤ʤɤνԤɬפޤ
# </p>
#===============================================================================
sub create_archive {
	my $file = shift;
	my @dirs = @_;
	my @files;
	
	foreach my $dir (@dirs){
		_push_files(\@files,$dir);
	}
	
	eval("use Archive::Tar;");
	Archive::Tar->create_archive($file,0,@files);
}

#===============================================================================
# compressؿ椫ƵŪ˸ƤФprivateʴؿ
#===============================================================================
sub _push_files {
	my $list = shift;
	my $dir  = shift;
	push(@$list,$dir);
	
	if(-d $dir){
		foreach my $entry (glob("$dir/*")){
			_push_files($list,$entry);
		}
	}
}

1;
